import chat.LoginWindow;
import chat.MessageBox;
import chat.UserListCell;
import org.aswing.JFrame;
import org.aswing.JTextArea;
import org.aswing.JTextField;
import org.aswing.JButton;
import org.aswing.JList;
import org.aswing.GridLayout;
import org.aswing.BorderLayout;
import org.aswing.JPanel;
import org.aswing.JLabel;
import org.aswing.JWindow;
import org.aswing.JScrollPane;
import org.aswing.Insets;
import org.aswing.Event;
import org.aswing.VectorListModel;
import org.aswing.GeneralListCellFactory;
import org.aswing.util.Delegate;
import org.aswing.border.LineBorder;
import org.aswing.border.EmptyBorder;
import it.gotoandplay.smartfoxserver.SmartFoxClient;
import it.gotoandplay.smartfoxserver.User;
import it.gotoandplay.smartfoxserver.Room;




class chat.Main
{
	/******
	* Components for the application
	********/
	
	//Main window application
	private var _mainFrame_frm:JFrame;
	//Main chat window
	private var _chatArea_ta:JTextArea;
	//User input messages
	private var _msg_txt:JTextField;
	//Displays all current users logged on
	private var _userList_lb:JList;
	//Stores names of all user lists
	private var _userList:VectorListModel;	
	//Sends the typed message
	private var _send_btn:JButton;
	//Display label to identify what the list control is about.
	private var _users_lbl:JLabel;
	//internal message window
	private var _messageWindow:JFrame;
	//instance of loginWindow;
	private var _loginWindow:LoginWindow;
	//to make the main chat area scrollable
	private var _mainChat_sp:JScrollPane;
	
	
	
	/*
	* Variables 
	*/
	
	//Stores the IP Address
	private var _ip:String;
	//Stores the port number
	private var _port:Number;
	//Stores the zone ( i.e which contains all the rooms )
	private var _zone:String;
	//Stores the current room
	private var _room:String;
	//Client object for the server
	private var _smartFoxClient:SmartFoxClient;
	//Id to generate Error message window
	private var ERROR_MESSAGE:Number=0;
	//id to generate Info message window
	private var INFO_MESSAGE:Number=1;
	//Stores the username
	private var _username:String;
	//FOR DEBUGGING
	private static var DEBUG_FLAG=true;
	
	
	
	public function Main(Void){
		init();
		buildGUI();
	}
	
	private function init(Void):Void{	
		//initialze the vector list model
		_userList=new VectorListModel();		
		//set the server name
		_ip="127.0.0.1"
		//set the port number
		_port=9339;
		//set the zone
		_zone="simpleChat";
		//Initialize the client 
		_smartFoxClient=new SmartFoxClient();
		//Trace statements out
		_smartFoxClient.debug=true;
		//Delegate method to handle connection
		_smartFoxClient.onConnection=Delegate.create(this,handleConnection);
		//Delegate method when login is successfull
		_smartFoxClient.onLogin=Delegate.create(this,onLogin);
		//handles the join room response i.e when the user joins a room
		_smartFoxClient.onJoinRoom=Delegate.create(this,onJoinRoom);
		//handles the response when the list of rooms is updated.
		_smartFoxClient.onRoomListUpdate=Delegate.create(this,onRoomListUpdate);
		//handles the response when a new user enters the room.
		_smartFoxClient.onUserEnterRoom=Delegate.create(this,onUserEnterRoom);
		//handles the response when a user leaves the room.
		_smartFoxClient.onUserLeaveRoom=Delegate.create(this,onUserLeaveRoom);
		//handles the response when a public message has been posted.
		_smartFoxClient.onPublicMessage=Delegate.create(this,onPublicMessage);	
		
	}
	
	private function buildGUI(Void):Void{	
		
		
		//Create main application window
		_mainFrame_frm=new JFrame(null,"ASWing Chat",true);
		//Set size for main application window
		_mainFrame_frm.setSize(799,599);
		//Make window visible
		_mainFrame_frm.setVisible(true);		
		//Make main app window not closable
		_mainFrame_frm.setClosable(false);
		//Make window not draggable
		_mainFrame_frm.setDragable(false);	
		//Make the window not resizable
		_mainFrame_frm.setResizable(false);
		//Set it to be active
		_mainFrame_frm.setActive(true);
		//Set new layout 
		_mainFrame_frm.getContentPane().setLayout(new BorderLayout(5,5));
		
		//Create new login window for user
		_loginWindow = new LoginWindow(_mainFrame_frm,"Login Window " , true);
		//Create a call back function for when the user clicks on the submit button.
		_loginWindow.addEventListener(LoginWindow.ON_SUBMIT,clearLoginWindow,this);
		
		
		//Create main chat window		
		_chatArea_ta=new JTextArea("Welcome to ASWing Chat\n---------------------------------------------",100,100);
		//Make main chat window not editable
		_chatArea_ta.setEditable(false);
		//Make the main chat window render html tags.
		_chatArea_ta.setHtml(true);
		//enable word wrap
		_chatArea_ta.setWordWrap(true);
		
		//create scroll pane for main chat area 
		_mainChat_sp=new JScrollPane();
		//set the view  ( i.e _chatArea_ta ) for the scroll pane 
		_mainChat_sp.setView(_chatArea_ta);
		//set scrollbar properties for the scrollpane
		_mainChat_sp.setVerticalScrollBarPolicy(JScrollPane.SCROLLBAR_AS_NEEDED);
		
	
		//Textfield to input messages
		_msg_txt=new JTextField("< Type message here > ");
		//create a call back function to send the message when the user presses the Enter key.
		_msg_txt.addEventListener(JTextField.ON_KEY_DOWN,onEnterSendMsg,this);
		//List control to hold all users currently logged on.
		_userList_lb=new JList(_userList);
		//Set the size of the list control
		_userList_lb.setPreferredSize(150,300);
		//Create display label for list control
		_users_lbl=new JLabel("User List");
		//Set alignment for label
		_users_lbl.setHorizontalAlignment(JLabel.LEFT);
		//create border
		_userList_lb.setBorder(new LineBorder(new EmptyBorder(null,new Insets(8,8,8,8))));
		//set the List Cell Renderer to our custom class.
		_userList_lb.setCellFactory(new GeneralListCellFactory(UserListCell));
		//Create submit button
		_send_btn=new JButton("Send");
		//Set size for the button
		_send_btn.setPreferredSize(100,22);
		//create a call back function for when the user sends a message to the server.
		_send_btn.addEventListener(JButton.ON_CLICKED,sendMessage,this);
		
		//Create center panel to hold main chat window
		var centerPane:JPanel=new JPanel();
		//set layout to BorderLayout with horizontal and vertical gaps set to 5.
		centerPane.setLayout(new BorderLayout(5,5));
		//add the main chat window to the center.
		centerPane.append(_mainChat_sp,BorderLayout.CENTER);
		
		//Right pane to hold list control and the label
		var rightPane:JPanel=new JPanel();
		//set layout to BorderLayout with horizontal and vertical gaps set to 5.
		rightPane.setLayout(new BorderLayout(5,5));
		//add the list control to the center
		rightPane.append(_userList_lb,BorderLayout.CENTER);
		//add the label on the top
		rightPane.append(_users_lbl,BorderLayout.NORTH);
		
		//Bottom pane to hold textfield and button
		var bottomPane:JPanel=new JPanel();
		//set layout to BorderLayout with horizontal and vertical gaps set to 5.
		bottomPane.setLayout(new BorderLayout(5,5));
		//add the user input textfield in the center
		bottomPane.append(_msg_txt,BorderLayout.CENTER);
		//add the send button to the left of the text field
		bottomPane.append(_send_btn,BorderLayout.EAST);
		//add the bottomPane below the main chat text area.
		centerPane.append(bottomPane,BorderLayout.SOUTH);		
		
		//Adding panels to main window
		_mainFrame_frm.getContentPane().append(centerPane,BorderLayout.CENTER);
		_mainFrame_frm.getContentPane().append(rightPane,BorderLayout.EAST);			
		
	}
	
	private function onEnterSendMsg(eventObj:Event):Void{
		//Check if the user hit the Enter key. If so send the message to the server.		
		if(Key.getCode()==Key.ENTER){
			_smartFoxClient.sendPublicMessage(_msg_txt.getText());
			//clear the input text field.
			_msg_txt.setText("");
		}
	}
	
	private function clearLoginWindow(eventObj:Event):Void{			
		//store the user name
		_username=_loginWindow.getUserName();		
		//remove the login window
		_loginWindow.destroy();	
		//Establish connection to the server
		_smartFoxClient.connect(_ip,_port);
	}
	
	private function addToChatWindow(msg:String):Void{
		//store the previous chat		
		var history:String=_chatArea_ta.getText()+"\n";		
		//add the history and new message.
		_chatArea_ta.setText(history +msg);
		//scroll to the bottom of the text area.
		_chatArea_ta.scrollToBottomRight();
		
	}
	
	private function sendMessage(eventObj:Event):Void{
		//send the message to the server as a public message
		_smartFoxClient.sendPublicMessage(_msg_txt.getText());
		//clear the input text field
		_msg_txt.setText("");
	}	
	
	private function handleConnection(success:Boolean):Void{
		//Create a new message box
		var msgBox:MessageBox=new MessageBox(_mainFrame_frm,"Server Message",null,true);		
		
		if(success){			
			//if the connection was successful
			msgBox.setMessage("Connection to the server has been established.");			
			//send the login information ( i.e zone name and user name) to the server.
			_smartFoxClient.login(_zone,_username);			
		}
		else{
			//if the connection failed.
			msgBox.setMessage("Failure to connect to the server.");						
		}		
		//show the messagebox
		msgBox.setVisible(true);
	}
	
	private function onLogin(success:Boolean):Void{		
		if(success){			
			
		}
		else{
			//if user could not login to the server.
			var msgBox:MessageBox=new MessageBox(_mainFrame_frm,"Server Message","Failure to log on to the server.",true);			
			//show the messagebox
			msgBox.setVisible(true);
		}		
	}
	
	private function onRoomListUpdate(roomList:Object):Void{
		//Auto Join the default room.
		_smartFoxClient.autoJoin();
	}
	
	
	private function onJoinRoom(roomObj:Room):Void{
		//clear all the users from the list.
		_userList.clear();		
		
		//store the list of all possible users in the room.
		var userList:Object=roomObj.getUserList();
		
		//Add all the possible user objects to the list control
		for (var name:String in userList){
			var user:User=userList[name];			
			_userList.append(user);			
		}
		
		//Sort the user names
		_userList.sort(Array.DESCENDING);
		
		
	}
	
	private function onUserEnterRoom(fromRoom:Number,usr:User):Void{
		//Show the message that a new user has entered the room.
		addToChatWindow("<font color='#cc0000'>>> Room [ " + usr.getName() + " ] joined</font>");		
		//Append the user name to the listbox component.
		_userList.append(usr);		
		//Sort the user names
		_userList.sort(Array.DESCENDING);		
	}
	
	private function onUserLeaveRoom(fromRoom:Number,usrId:Number):Void{
		//Variable to store the user name.
		var userName:String;
		
		// Iterate through all the user names 
		for ( var count:Number=0; count < _userList.getSize();count++){
			//Store the current user object.
			var user:User=User(_userList.getElementAt(count));
			//Check if the user ids match
			if(user.getId()==usrId){
				//Store the name of the user that left the room.s
				userName=String(_userList.getElementAt(count));	
				//Remove the user from the list control.
				_userList.removeAt(count);
				//stop the loop
				break;
			}
		}
		//Show message , which user left the room.
		addToChatWindow("<font color='#cc0000'>>> Room [ " + userName + " ] left the room.</font>");			
		//Sort the user names
		_userList.sort(Array.DESCENDING);		
		
	}
	
	private function onPublicMessage(msg:String,usr:User):Void{
		//Add the public message to the main chat text area.
		addToChatWindow("<b>[ " + usr.getName() + " ]:</b> " + msg);
	}	
	
	public static function DEBUG(msg:String):Void{
		//If DEBUG_FLAG is set to true , trace statements out.
		if(DEBUG_FLAG){
			trace("DEBUG => " +msg);
		}
	}	
	
	public static function main(Void):Void
	{
		//Start the application
		var app:Main=new Main();
	}
}
